<?php

namespace Drupal\override_cache_control_headers\EventSubscriber;

use Drupal\Core\Config\ConfigFactoryInterface;
use Drupal\Core\State\StateInterface;
use Drupal\override_cache_control_headers\Utility;
use Symfony\Component\EventDispatcher\EventSubscriberInterface;
use Symfony\Component\HttpKernel\Event\ResponseEvent;
use Symfony\Component\HttpKernel\KernelEvents;

/**
 * Set the configured cache control headers against the provided URL.
 */
class OverrideCacheControlHeaders implements EventSubscriberInterface {

  /**
   * The config factory.
   *
   * @var \Drupal\Core\Config\ConfigFactoryInterface
   */
  protected $configFactory;

  /**
   * The state service.
   *
   * @var \Drupal\Core\State\StateInterface
   */
  protected $state;

  /**
   * The utility service.
   *
   * @var \Drupal\override_cache_control_headers\Utility
   */
  protected $utility;

  /**
   * Override cache control header constructor.
   *
   * Initialize the config factory object.
   *
   * @param \Drupal\Core\Config\ConfigFactoryInterface $configFactory
   *   The configuration factory service.
   * @param \Drupal\Core\State\StateInterface $state
   *   The state service.
   * @param \Drupal\override_cache_control_headers\Utility $utility
   *   The utility service.
   */
  public function __construct(
    ConfigFactoryInterface $configFactory,
    StateInterface $state,
    Utility $utility,
  ) {
    $this->configFactory = $configFactory;
    $this->state = $state;
    $this->utility = $utility;
  }

  /**
   * Modifies the response to disable cache for a specific route.
   *
   * @param \Symfony\Component\HttpKernel\Event\ResponseEvent $event
   *   The response event that contains the request and response objects.
   */
  public function onViewResponse(ResponseEvent $event) {
    // Check if the request path matches the specific route.
    $urls_header = $this->configFactory->get('override_cache_control_headers.settings')->get('urls_header');
    $set_headers = [];
    if (!empty($urls_header)) {
      $urls_header = trim($urls_header);
      $urls_headers = explode(PHP_EOL, $urls_header);
      if ((is_array($urls_headers)) && (!empty($urls_headers))) {
        $set_headers = array_filter($urls_headers);
      }
    }
    $urls_header = $this->utility->getTempHeadersInState();
    if (!empty($urls_header)) {
      $state_changed = FALSE;
      $urls_header = $this->utility->processTempUrlHeaders($urls_header, $state_changed);
      if ($state_changed) {
        $this->utility->resetTempHeadersInState($urls_header);
      }
      $set_headers = array_merge($set_headers, $urls_header);
    }
    if (!empty($set_headers)) {
      $this->setHeaders($set_headers, $event);
    }
  }

  /**
   * Sets the headers for the given URLs.
   *
   * @param array $urls_headers
   *   An array of URLs for which the headers need to be set.
   * @param \Symfony\Component\HttpKernel\Event\ResponseEvent $event
   *   The response event that contains the request and response objects.
   */
  private function setHeaders(array $urls_headers, ResponseEvent $event) {
    $pathinfo = $event->getRequest()->getPathInfo();
    foreach ($urls_headers as $urls_header) {
      $exploded = explode('|', $urls_header);
      if ($pathinfo === $exploded[0]) {
        $response = $event->getResponse();
        // Set Cache-Control header to disable.
        // caching for this specific route.
        $response->headers->set('Cache-Control', $exploded[1]);
        return;
      }
    }
  }

  /**
   * {@inheritdoc}
   */
  public static function getSubscribedEvents(): array {
    $events = [];
    $events[KernelEvents::RESPONSE][] = ['onViewResponse', 10000];
    return $events;
  }

}
